#ifndef __Custom_H__
#define __Custom_H__

#include "avisynth.h"
#include "x264/extras/stdint.h"

extern "C" {

#include "x264/x264.h"

}

#include <list>

namespace CustomMatrices {

typedef unsigned char Byte;

typedef struct {

   bool  isInter;
   bool  isBidir;

   bool  isCbr;
   bool  isDeblocking;
   bool  is8x8;

   int   nBitrate;
   int   nQuantizer;
   int   nDeblockingStrength;

   bool  showMetrics;
   bool  showSize;

   const char *pszMatrices[6];
   const char *pszMatricesFile;

   int   nMe;
   int   nAq;

   float fAq;

} Configuration;

typedef std::pair< int, int64_t > Pair;

class Queue : public std::list< Pair >
{
   int64_t nLength;
   int nNumerator;
public:
   Queue(int64_t nLength, int nNum) : nLength(nLength), nNumerator(nNum) {}
   void push(int nValue, int64_t ts)
   {
      push_back( Pair( nValue, ts * 1000 / nNumerator ) );
      clean();
   }
   int64_t diff()
   {
      return back().second - front().second;
   }
   void clean()
   {
      while ( diff() > nLength )
         pop_front();
   }
   int sum()
   {
      int nSum = 0;
      for ( const_iterator it = begin(); it != end(); it++ )
         nSum += it->first;

      return nSum;
   }
   int bitrate()
   {
      if ( !diff() )
         return 0;
      return (int64_t)sum() * 1000 * 8 / diff();
   }
};

class Filter : public GenericVideoFilter
{
   int nWidth, nHeight;

   bool showSize;
   bool showPsnr;

   x264_t *x264;

   x264_picture_t input;
   x264_picture_t output;

   Configuration configuration;

   int nFrames;

   Queue short_term, long_term;
   
public:
   Filter(PClip child, const Configuration &configuration, IScriptEnvironment *env);
   ~Filter();

   PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env);

};

}

#endif
